--引导
Lua_GuideWindow = class("Lua_GuideWindow", Lua_Window);
function Lua_GuideWindow:ctor()
    self:superCall("ctor");
	self.blackAlpha = 0.01;
	self.maskSprite = "black_mask";
	self.showClose = false;
	self.showBlackMask = true;
	self.layer = MainWinDefine.GUIDE_WINDOW_LAYER;
	self.windowName = "Lua_GuideWindow";
	self.skinName = "GuideWindow_skin";

	self:addPreLoadRes("/images/timeline/window_bg_1.png", Lua_LoadFileType.IMG);
	self:addPreLoadRes("/images/timeline/show_iamge_1.png", Lua_LoadFileType.IMG);
	self:addPreLoadRes("/images/timeline/show_iamge_2.png", Lua_LoadFileType.IMG);
end

function Lua_GuideWindow:buildPanel(go)
	self:superCall("buildPanel", go);
	self:initEvent();
end

function Lua_GuideWindow:initEvent()
	self:addLuaEvent("PLAY_GUIDE", self, self.playGuide);--播放引导
	self:addLuaEvent("TEST_PLAY_GUIDE_STEP", self, self.onPlayStep);--测试工具回调步骤id
	self:addLuaEvent("START_GUIED_STEP", self, self.startGuideStep);--接收下一步引导id状态

	--timeLine播放器回调
	self:addCSharpEvent("TIME_LINE_PLAY_END", self, self.onPlayEnd);--播放timeline结束
	self:addCSharpEvent("TIME_LINE_PLAY_STEP", self, self.onPlayStep);--播放器回调步骤id
	self:addCSharpEvent("TIME_LINE_CONTEN_END", self, self.onContentEnd);--当前对话播放完毕
	self:addCSharpEvent("TIME_LINE_PLAY_MASK_END", self, self.onPlayMaskEnd);--播放过渡遮罩结束
end

function Lua_GuideWindow:buildShowItem()
	self:superCall("buildShowItem");
	self.guide_id = 0;
	self.special_type = 0;--特殊事件类型
	self.bOnMask = false;--点击背景限制
	self.bAutoClick = false;--是否自动
	self.bHideClick = false;--是否隐藏
	self.bPauseState = false;--暂停状态
	self.bAutoPlayTimer = false;--自动播放下一步计时器状态

	self.content:SetActive(false);
	self.dialogue_obj:SetActive(false);
	self.jumpBtn:addClickFuntion(self, self.onJumpBtnClick);--跳过
	self.lockBtn:addClickFuntion(self, self.onLockBtnClick);--回顾
	self.autoBtn:addClickFuntion(self, self.onAutoBtnClick);--自动
	self.hideBtn:addClickFuntion(self, self.onHideBtnClick);--隐藏
	self.clickBtn:addClickFuntion(self, self.onMaskBgClick);--遮罩
end

function Lua_GuideWindow:showWin(param)
	self:superCall("showWin",param);
	self.guide_id = param;
	self.gudieVo = DataCenter.guideData:getGuideVoById(param);
	if self.gudieVo == nil then
		self:hide();
		return;
	end
	self.bOnMask = true;--timeline资源未加载，屏蔽点击
	self:preLoadRes();
end

--预加载资源
function Lua_GuideWindow:preLoadRes()
    local vo = Lua_PreLoadResInfo.new();
    vo.resType = Lua_LoadFileType.PLAYABLE;-- 资源类型
    vo.resName = self.gudieVo.path; -- 资源名
    local arr = Array();
    arr:add(vo);
    DataCenter.preLoadData:preLoad("Lua_GuideWindow", Lua_PreLoadDefine.LOADSHOW_TYPE_NONE, self, self.onGuidePlayShow, nil, arr);
end

function Lua_GuideWindow:onGuidePlayShow()
	self.bOnMask = false;

	if self.gudieVo.path == nil then
		return;
	end
	local roleName = DataCenter.roleData.name;
	TimeLineUtil.getInstance():InitDirector(self.content, roleName);

	self:specialEvent();
end

--特殊事件处理
function Lua_GuideWindow:specialEvent()
	if self.guide_id == DataCenter.guideUtil.create_role_gid1 then
		--print("创角引导等待过场动画播放完毕，再开始引导")
	else
		if self.gudieVo.bSpecial then
			local step_id = self.guide_id*1000 + 1;--引导第一步
			self.stepVo = DataCenter.guideUtil:getStepVoById(step_id);
			if self.stepVo ~= nil then
				if self.stepVo.special_type == 1 then
					local bool = ContainerUtil.getObjState("uiContainer", self.stepVo.clickObj);
					if bool == false then--没找到对象
						self.special_type = 1;
						--特殊情况，需要强制打开指定窗口，检测到点击UI路径，然后在开始播放引导
						-- print("特殊情况 = =",self.stepVo.funcPathId)
						if self.stepVo.funcPathId ~= nil then
							local pathData = Lua_FuncPathData.getConfigData(self.stepVo.funcPathId); -- 跳转冒险
							if pathData ~= nil then
								pathData:gotoFunction(true);
							end
							self:checkObjectState(self.stepVo.clickObj);--检测点击UI路径
						end
					else
						self:playGuide();--没特殊情况直接播放引导
					end
				else
					self:playGuide();--没特殊情况直接播放引导
				end
			end
		else
			self:playGuide();--没特殊情况直接播放引导
		end
	end
end

--播放引导
function Lua_GuideWindow:playGuide()
	self.maskBg.alpha = 0.01;
	local path = UrlManager.fixFileSuffix(self.gudieVo.path);
	TimeLineUtil.getInstance():Play(path);
end

--播放引导步骤
function Lua_GuideWindow:onPlayStep(step)
	self.bPauseState = false;
	self.click_obj:SetActive(false);--特效点击父节点
	self.continue_obj:SetActive(false);--继续按钮父节点
	self:updateMask(step);
end

--更新遮罩
function Lua_GuideWindow:updateMask(step)
	if step == nil and step == 0 then
		return
	end
	self.bOnMask = false;
	self.step_id = self.guide_id*1000 + step;
	self.stepVo = DataCenter.guideUtil:getStepVoById(self.step_id);
	if self.stepVo == nil then
		return;
	end
	self.gudieVo.clientStep = self.stepVo.step;--设置当前引导步骤
	
	local gameMode = self.stepVo.gameMode;
	if self.gameMode ~= gameMode then
		self.gameMode = gameMode;
		self:updateComponents();
	end

	self:creatEffect();
	self:updateShowBtn();
	self.content:SetActive(true);
	local contentStr = TimeLineUtil.getInstance().curContentStr;--描述
	self.dialogue_obj:SetActive(contentStr ~= "");

	local location = self.stepVo.location;
	local dimensions = self.stepVo.dimensions;
	self.clickBtn:SetDimensions(dimensions[1],dimensions[2]);
	LuaUtil.setObjPos(self.click_obj, location[1], location[2], 0);
	self.showIcon:setLocation(25, -40, 0);
	TweenUtil.getInstance():stop(self.showIcon.mainContent);

	if self.gameMode == 1 then --引导模式
		self:endAutoTime();
		if self.stepVo.mask_type == -1 then
			self:creatMaskObj(false);
			self.maskBg:setEnable(false);
		else
			self:creatMaskObj(true);
			self.maskBg:setEnable(true);
		end
		
		self.winBg:setEnable(false);
		self.image:setEnable(false);
		self.sprite:setEnable(false);
		self.click_obj:SetActive(true);
	else--剧情模式
		if self.contentStrArr == nil then
			self.contentStrArr = Array();
		end
		if contentStr ~= nil and contentStr ~= "" then
			self.contentStrArr:add({name = self.show_name.text, des = contentStr});
		end
		self:creatMaskObj(false);
	end

	if self.stepVo.operate_type == 4 then--拖动箭头
		self:showDrang();
	end

	if self.stepVo.move_loction ~= nil then--移动效果效果
		self:showMoveTo();
	end
	self:starAutoPlayTimer();--自动开始下一步
end

--创建特效
function Lua_GuideWindow:creatEffect()
	if self.showEff == nil then--特效展示
		self.showEff = UICreate.createUIEffect(self.click_obj, "/effects/scsEffects/gui_zhiyin1", "gui_zhiyin1", 0, 0);--常驻特效
		self.showEff:setEffSca3D(self.stepVo.effSca[1], self.stepVo.effSca[2], self.stepVo.effSca[3]);
		self.showEff.siblingIndex = 0;
		self.clickEff = UICreate.createUIEffect(self.click_obj, "/effects/scsEffects/gui_zhiyin2", "gui_zhiyin2", 0, 0);--点击特效
		self.clickEff:setEffSca3D(280, 280, 280);
		self.clickEff.siblingIndex = 1;
	end
end

--创建点击区域遮罩
function Lua_GuideWindow:creatMaskObj(bool)
	if bool == false then
		if self.maskObj ~= nil then
			self.maskObj:setEnable(false);
		end
		return
	end

	if self.maskObj == nil then
		self.maskObj = true;
		self.maskObj = UICreate.createMask(self.content, 0, 0, 0, 0);--遮罩
		self.maskObj.siblingIndex = 0;
		self.maskObj:addClickFuntion(self, self.onMaskObjClick);
		self.maskObj:SetDimensions(720,1560);
	end

	local location = self.stepVo.location;
	local dimensions = self.stepVo.dimensions;
	self.maskObj:setEnable(true);
	self.maskObj:setColor(0,0,0,self.stepVo.mask_alpha*255);
	self.maskObj:setRectShrinkValue(self.stepVo.mask_time, 720, 1280);
	self.maskObj:setTargetRectInfo(location[1], location[2], dimensions[1]/2, dimensions[2]/2, self.stepVo.mask_type);
end

--文本播放完毕，展示继续
function Lua_GuideWindow:onContentEnd()
	if self.gameMode == 2 then --剧情
		self:starAutoTime();
		self.bPauseState = true;
		self.click_obj:SetActive(true);
		self.continue_obj:SetActive(true);
	end
end

--拖动效果展示
function Lua_GuideWindow:showDrang()
	if self.arrows == nil then
		self.arrows = true;
		self.arrows = UICreate.createUIAtlasImage(self.content, "common_altas_1", "ui_common_label_16", 0, 0, 0, 0);
		self.arrows:setRot(0, 0, -135);
	end
	local location = self.stepVo.arrows_loction;
	self.arrows:setLocation(location[1], location[2], 0);
end

--UI移动效果
function Lua_GuideWindow:showMoveTo()
	local move_loction = self.stepVo.move_loction;
	self.showIcon:setLocation(25, -40, 0);
	TweenUtil.getInstance():stop(self.showIcon.mainContent);
	TweenUtil.getInstance():addTweenTargetPos(self.showIcon.mainContent, 2, 0, move_loction[1], move_loction[2], 0, self, self.showMoveTo);
end

--更新组件
function Lua_GuideWindow:updateComponents()
	if self.gameMode == 1 then--引导
		self.jumpBtn:setLocation(270, 621, 0);
		self.show_name:setEnable(false);
        self.show_content:setLocation(40, 7, 0);
        --self.show_content:SetDimensions(365, 60);
        --self.show_role:setAltas("arena_atlas_1", "ui_djc_rw_01");
        self.show_role:setEnable(true);
	elseif self.gameMode == 2 then--剧情对话
		self.jumpBtn:setLocation(293, -148, 0);
		self.show_role:setEnable(false);
        self.show_name:setEnable(true);
        self.show_content:setLocation(-350, 10, 0);
        self.show_content:SetDimensions(700, 60);
	end
end

--更新按钮展示
function Lua_GuideWindow:updateShowBtn()
	self.jumpBtn:setEnable(false);
	self.lockBtn:setEnable(false);
	self.autoBtn:setEnable(false);
	self.hideBtn:setEnable(false);

	local list = self.stepVo.btnList;
	local len = #list
	for i = 1, len do
		if list[i] == 1 then -- 跳过
			self.jumpBtn:setEnable(true);
		elseif list[i] == 2 then -- 回顾
			self.lockBtn:setEnable(true);
		elseif list[i] == 3 then -- 隐藏
			self.hideBtn:setEnable(true);
		elseif list[i] == 4 then -- 自动
			self.autoBtn:setEnable(true);
		end
	end

	self.btn_obj:SetActive(len >0);
end

--点击引导遮罩
function Lua_GuideWindow:onMaskObjClick()
	-- print("点击引导遮罩")
	self.clickEff:setEnable(false);
	self.clickEff:setEnable(true);
end

--点击引导遮罩按钮
function Lua_GuideWindow:onMaskBgClick()
	self:endAutoPlayTimer();--关闭自动播放下一步骤
	local type = self.stepVo.condition_type;
	if type == 0 then--无需条件
		self:onPlayGuideStep();
	elseif type == 1 then--功能内部特殊条件(需引导层和外部交互)
		Lua_MsgUtil.callFunc("PLAY_GUIED_STEP", self.step_id);
	else--指定条件
		local state = DataCenter.guideData:checkConditionState(self.step_id);
		if state then
			self:onPlayGuideStep();
		else
			self:hide();
		end
	end
end

--外部条件返回 引导步骤状态
function Lua_GuideWindow:startGuideStep(param)
	if param == nil then
		return;
	end
	local step_id = param.step_id;
	local paly_state = param.paly_state;
	if self.step_id ~= step_id then
		return;
	end
	
	if paly_state then
		self:onPlayGuideStep();
	else
		self:hide();
	end
end

--播放引导步骤
function Lua_GuideWindow:onPlayGuideStep()
	if self.gameMode == 2 and self.bPauseState == false then
		TimeLineUtil.getInstance():SkipTimeLine();--跳过
		return
	end

	DataCenter.guideData:recordGuide(self.guide_id);--记录设置当前引导步骤
	--有需网络波动校验在这里加
    DataCenter.guideUtil:gotoStepFunction(self.step_id);--执行点击事件
	self:checkObjectState();--检测下一步需要打开的界面是否加载完
end

--播放引导结束
function Lua_GuideWindow:onPlayEnd()
	if self.gudieVo.next_guide_id ~= nil then
		self.content:SetActive(false);
		self:clen();
		if DataCenter.guideData:checkOpenGuide() then--有新的引导，就播放新的引导
			
		else--不满足开启新的引导
			self:hide();
		end
	else
		self:hide();
	end
end

--播放过渡遮罩结束
function Lua_GuideWindow:onPlayMaskEnd(resource_id)
    self:onPlayEnd();
end













-------------------------------------------- 自动开始下一步 --------------------------------------------

--自动播放下一步倒计时
function Lua_GuideWindow:starAutoPlayTimer()
	self.countTime = self.stepVo.auto_time;
	if self.countTime ~= nil then
		if self.bAutoPlayTimer == false then
			self.bAutoPlayTimer = true;
			self:addTimer( "autoPlayTimer", 1, self, self.autoPlayTimer, nil, -1);
		end
	else
		self:endAutoPlayTimer();
	end
end

function Lua_GuideWindow:autoPlayTimer()
	self.countTime = self.countTime - 1;
	if self.countTime <= 0 then
		self:endAutoPlayTimer();
		self:onMaskBgClick();--自动开始下一步
	end
end

function Lua_GuideWindow:endAutoPlayTimer()
    if self.bAutoPlayTimer == true then
		self.bAutoPlayTimer = false;
		self:removeTimer(self, self.autoPlayTimer);
	end
end








-------------------------------------------- 目标检测 --------------------------------------------

--检测下一个点击对象是否加载
function Lua_GuideWindow:checkObjectState(path)
	if path == nil then
		self.checkObj = self.stepVo.checkObj;
	else
		self.checkObj = path;
	end
    if self.checkObj ~= nil then
    	self.bOnMask = true;--timeline未播放到下一步，屏蔽点击
    	self.content:SetActive(false);

    	self.count = 0;--检测计数
    	self.starTime = getTime();
    	self.checkTime = self.stepVo.check_time;--检测超时
    	self.checkCount = DataCenter.guideUtil.check_count;--检测次数
    	self:addTimer( "updateCheckObject", 0.01, self, self.updateCheckObject, nil, -1);
    else
	    self.image:setEnable(self.stepVo.bEnable);
		self.dialogue_obj:SetActive(self.stepVo.bEnable);

    	self:endCheckObject();
    	self:starNextGuideStep();
    end
end

function Lua_GuideWindow:updateCheckObject()
	if self.count <= self.checkCount then--检测次数
	    local bool = ContainerUtil.getObjState("uiContainer", self.checkObj);
		if bool == true then
			self.count = self.count + 1;
		else
			if getTime() - self.starTime >= self.checkTime then
				LogManager.LogError("检测目标对象超时    关闭引导 步骤 =  = "..self.checkObj, false);
				self:endCheckObject();
				self:hide();
			end
		end
	else
		self:endCheckObject();
    	self:starNextGuideStep();
	end
end

function Lua_GuideWindow:endCheckObject()
    self:removeTimer(self,self.updateCheckObject);
end


--开始执行下一步引导
function Lua_GuideWindow:starNextGuideStep()
	-- print("开始执行下一步引导",self.special_type)
	if self.special_type == 1 then
		self.special_type = 0;
		-- print("检测到目标对象，开始引导",self.checkObj)
		self:playGuide();
	else
		local state;
		local goto_time;
		if self.stepVo.goto_step ~= nil then
			goto_time = DataCenter.guideData:checkStepTime(self.gudieVo.id, self.gudieVo.serverStep);
		end
		-- print(goto_time, "开始执行下一步引导" )
		if goto_time == nil then
	    	TimeLineUtil.getInstance():ResumeTimeLine();
	    else
	    	TimeLineUtil.getInstance():GotoTimeLine(goto_time);
	    end
	end
end


















-------------------------------------------- 按钮点击 --------------------------------------------

--开启自动播放计时器
function Lua_GuideWindow:starAutoTime()
    if self.bAutoClick == true then
    	self.starTime = DataCenter.guideUtil.auto_stop_time;
        Lua_CommonUtil.addTimer( "updateAutoTime", 1, self, self.updateAutoTime);
    else
        self:endAutoTime();
    end
end

--关闭自动播放计时器
function Lua_GuideWindow:endAutoTime()
	if self.bAutoClick == false then
		return;
	end
	self.bAutoClick = false;
    Lua_CommonUtil.removeTimer(self,self.updateAutoTime);
end

function Lua_GuideWindow:updateAutoTime()
    if self.starTime > 0 then
    	self.starTime = self.starTime -1;
    else
    	Lua_CommonUtil.removeTimer(self,self.updateAutoTime);
    	self:onPlayGuideStep();
    end
end

--跳过剧情
function Lua_GuideWindow:onJumpBtnClick()
	self:onPlayMaskEnd();
end

--回顾
function Lua_GuideWindow:onLockBtnClick()
	if self.bAutoClick == true then
		-- self.autoBtn:setAltas();
		self:endAutoTime();
		OperateAlert.showLangValue(370001);
	end
	Lua_WindowUtil.openWin("Lua_GuideLockWindow", Lua_WinParam.ACTION_OPEN, self.contentStrArr);
end

--隐藏
function Lua_GuideWindow:onHideBtnClick()
	if self.bHideClick == false then
		self.bHideClick = true;
		self.content:SetActive(false);
	end
end

--自动
function Lua_GuideWindow:onAutoBtnClick()
	if self.bAutoClick == false then
		self.bAutoClick = true;
		-- self.autoBtn:setAltas();
		OperateAlert.showLangValue(370000);
		if self.bPauseState == true then--当前对话播放完毕
			self:starAutoTime();
		end
	else
		self:endAutoTime();
		-- self.autoBtn:setAltas();
		OperateAlert.showLangValue(370001);
	end
end

--背景遮罩
function Lua_GuideWindow:onMaskClick()
	-- print("背景遮罩")
	if self.bOnMask then--点击屏蔽
		return
	end
	if self.bHideClick == true then--隐藏
		self.bHideClick = false;
		self.content:SetActive(true);
	else
		self.click_obj:SetActive(false);
		self:onPlayGuideStep();
	end
end

--背景
function Lua_GuideWindow:onWinBgClick(go)
	-- print("点击背景")
	if self.bOnMask then--点击屏蔽
		return
	end
	if self.bHideClick == true then--隐藏
		self.bHideClick = false;
		self.content:SetActive(true);
	else
		self.click_obj:SetActive(false);
		self:onPlayGuideStep();
	end
end

function Lua_GuideWindow:clen()
	self.stepVo = nil;
	self.gudieVo = nil;
	if self.showIcon ~= nil then
		TweenUtil.getInstance():stop(self.showIcon.mainContent);
	end

	DataCenter.guideData:removeGuide(self.guide_id);
	TimeLineUtil.getInstance():dispose();
	self:endAutoTime();
	self:endCheckObject();
	self:endAutoPlayTimer();
end

function Lua_GuideWindow:dispose()
	self.maskObj = nil;
	self.showEff = nil;
	self.clickEff = nil;
	TimeLineUtil.getInstance():PauseTimeLine();
	self:clen();
	self:superCall("dispose");
end